@@ -1,4 +1,4 @@
-# Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0,
@@ -31,107 +31,107 @@ from workbench.utils import find_object_with_name, Version
 ModuleInfo = DefineModule(name= "DbMssqlMigration", author= "Oracle Corp.", version="1.0")
 
 # unfinished
-MSSQL_COLLATION_MAP= [
-('SQL_Latin1_General_Cp437_BIN', 'utf8_general_ci', False),
-('SQL_Latin1_General_Cp437_CS_AS', 'utf8_general_ci', False),
-('SQL_Latin1_General_Cp437_CI_AS', 'utf8_general_ci', False),
-('SQL_Latin1_General_Pref_CP437_CI_AS', 'utf8_general_ci', False),
-('SQL_Latin1_General_Cp437_CI_AI', 'utf8_general_ci', False),
-('SQL_Latin1_General_Cp850_BIN', 'utf8_bin', True),
-('SQL_Latin1_General_Cp850_CS_AS', 'utf8_general_ci', False),
-('SQL_Latin1_General_Cp850_CI_AS', 'utf8_general_ci', False),
-('SQL_Latin1_General_Pref_CP850_CI_AS', 'utf8_general_ci', False),
-('SQL_Latin1_General_Cp850_CI_AI', 'utf8_general_ci', False),
-('SQL_1Xcompat_CP850_CI_AS', 'utf8_general_ci', False),
-('Latin1_General_BIN', 'latin1_bin', True),
-('SQL_Latin1_General_Cp1_CS_AS', 'utf8_general_ci', False),
-('SQL_Latin1_General_Cp1_CI_AS', 'utf8_general_ci', True),
-('SQL_Latin1_General_Pref_CP1_CI_AS', 'utf8_general_ci', False),
-('SQL_Latin1_General_Cp1_CI_AI', 'utf8_general_ci', True),
-('SQL_AltDiction_Cp850_CS_AS', 'utf8_general_ci', False),
-('SQL_AltDiction_Pref_CP850_CI_AS', 'utf8_general_ci', False),
-('SQL_AltDiction_Cp850_CI_AI', 'utf8_general_ci', False),
-('SQL_Scandinavian_Pref_Cp850_CI_AS', 'utf8_swedish_ci', False),
-('SQL_Scandinavian_Cp850_CS_AS', 'utf8_swedish_ci', False),
-('SQL_Scandinavian_Cp850_CI_AS', 'utf8_swedish_ci', False),
-('SQL_AltDiction_Cp850_CI_AS', 'utf8_general_ci', False),
-('Latin1_General_CS_AS', 'utf8_general_ci', False),
-('Latin1_General_CI_AS', 'utf8_general_ci', False),
-('Danish_Norwegian_CS_AS', 'utf8_danish_ci', False),
-('Finnish_Swedish_CS_AS', 'utf8_swedish_ci', False),
-('Icelandic_CS_AS', 'utf8_icelandic_ci', False),
-('Hungarian_BIN', 'utf8_bin', True),
-('SQL_Latin1_General_Cp1250_CS_AS', 'cp1250_general_ci', False),
-('SQL_Latin1_General_Cp1250_CI_AS', 'cp1250_general_ci', True),
-('SQL_Czech_Cp1250_CS_AS', 'cp1250_czech_cs', True),
-('SQL_Czech_Cp1250_CI_AS', 'cp1250_czech_cs', False),
-('SQL_Hungarian_Cp1250_CS_AS', 'cp1250_general_ci', False),
-('SQL_Hungarian_Cp1250_CI_AS', 'cp1250_general_ci', False),
-('SQL_Polish_Cp1250_CS_AS', 'sql_polish_cp1250_cs_as', False),
-('SQL_Polish_Cp1250_CI_AS', 'sql_polish_cp1250_ci_as', False),
-('SQL_Romanian_Cp1250_CS_AS', 'sql_romanian_cp1250_cs_as', False),
-('SQL_Romanian_Cp1250_CI_AS', 'sql_romanian_cp1250_ci_as', False),
-('SQL_Croatian_Cp1250_CS_AS', 'sql_croatian_cp1250_cs_as', False),
-('SQL_Croatian_Cp1250_CI_AS', 'sql_croatian_cp1250_ci_as', False),
-('SQL_Slovak_Cp1250_CS_AS', 'sql_slovak_cp1250_cs_as', False),
-('SQL_Slovak_Cp1250_CI_AS', 'sql_slovak_cp1250_ci_as', False),
-('SQL_Slovenian_Cp1250_CS_AS', 'sql_slovenian_cp1250_cs_as', False),
-('SQL_Slovenian_Cp1250_CI_AS', 'sql_slovenian_cp1250_ci_as', False),
-('Cyrillic_General_BIN', 'cyrillic_general_bin', False),
-('SQL_Latin1_General_Cp1251_CS_AS', 'sql_latin1_general_cp1251_cs_as', False),
-('SQL_Latin1_General_Cp1251_CI_AS', 'sql_latin1_general_cp1251_ci_as', False),
-('SQL_Ukrainian_Cp1251_CS_AS', 'sql_ukrainian_cp1251_cs_as', False),
-('SQL_Ukrainian_Cp1251_CI_AS', 'sql_ukrainian_cp1251_ci_as', False),
-('Greek_BIN', 'greek_bin', False),
-('SQL_Latin1_General_Cp1253_CS_AS', 'sql_latin1_general_cp1253_cs_as', False),
-('SQL_Latin1_General_Cp1253_CI_AS', 'sql_latin1_general_cp1253_ci_as', False),
-('SQL_MixDiction_Cp1253_CS_AS', 'sql_mixdiction_cp1253_cs_as', False),
-('SQL_AltDiction_Cp1253_CS_AS', 'sql_altdiction_cp1253_cs_as', False),
-('SQL_Latin1_General_Cp1253_CI_AI', 'sql_latin1_general_cp1253_ci_ai', False),
-('Turkish_BIN', 'turkish_bin', False),
-('SQL_Latin1_General_Cp1254_CS_AS', 'sql_latin1_general_cp1254_cs_as', False),
-('SQL_Latin1_General_Cp1254_CI_AS', 'sql_latin1_general_cp1254_ci_as', False),
-('Hebrew_BIN', 'hebrew_bin', False),
-('SQL_Latin1_General_Cp1255_CS_AS', 'sql_latin1_general_cp1255_cs_as', False),
-('SQL_Latin1_General_Cp1255_CI_AS', 'sql_latin1_general_cp1255_ci_as', False),
-('Arabic_BIN', 'arabic_bin', False),
-('SQL_Latin1_General_Cp1256_CS_AS', 'sql_latin1_general_cp1256_cs_as', False),
-('SQL_Latin1_General_Cp1256_CI_AS', 'sql_latin1_general_cp1256_ci_as', False),
-('SQL_Latin1_General_Cp1257_CS_AS', 'sql_latin1_general_cp1257_cs_as', False),
-('SQL_Latin1_General_Cp1257_CI_AS', 'sql_latin1_general_cp1257_ci_as', False),
-('SQL_Estonian_Cp1257_CS_AS', 'sql_estonian_cp1257_cs_as', False),
-('SQL_Estonian_Cp1257_CI_AS', 'sql_estonian_cp1257_ci_as', False),
-('SQL_Latvian_Cp1257_CS_AS', 'sql_latvian_cp1257_cs_as', False),
-('SQL_Latvian_Cp1257_CI_AS', 'sql_latvian_cp1257_ci_as', False),
-('SQL_Lithuanian_Cp1257_CS_AS', 'sql_lithuanian_cp1257_cs_as', False),
-('SQL_Lithuanian_Cp1257_CI_AS', 'sql_lithuanian_cp1257_ci_as', False),
-('SQL_Danish_Pref_Cp1_CI_AS', 'sql_danish_pref_cp1_ci_as', False),
-('SQL_SwedishPhone_Pref_Cp1_CI_AS', 'sql_swedishphone_pref_cp1_ci_as', False),
-('SQL_SwedishStd_Pref_Cp1_CI_AS', 'sql_swedishstd_pref_cp1_ci_as', False),
-('SQL_Icelandic_Pref_Cp1_CI_AS', 'sql_icelandic_pref_cp1_ci_as', False),
-('Japanese_BIN', 'japanese_bin', False),
-('Japanese_CI_AS', 'japanese_ci_as', False),
-('Korean_Wansung_BIN', 'korean_wansung_bin', False),
-('Korean_Wansung_CI_AS', 'korean_wansung_ci_as', False),
-('Chinese_Taiwan_Stroke_BIN', 'chinese_taiwan_stroke_bin', False),
-('Chinese_Taiwan_Stroke_CI_AS', 'chinese_taiwan_stroke_ci_as', False),
-('Chinese_PRC_BIN', 'chinese_prc_bin', False),
-('Chinese_PRC_CI_AS', 'chinese_prc_ci_as', False),
-('Japanese_CS_AS', 'japanese_cs_as', False),
-('Korean_Wansung_CS_AS', 'korean_wansung_cs_as', False),
-('Chinese_Taiwan_Stroke_CS_AS', 'chinese_taiwan_stroke_cs_as', False),
-('Chinese_PRC_CS_AS', 'chinese_prc_cs_as', False),
-('Thai_BIN', 'thai_bin', False),
-('Thai_CI_AS', 'thai_ci_as', False),
-('Thai_CS_AS', 'thai_cs_as', False),
-('SQL_EBCDIC037_CP1_CS_AS', 'sql_ebcdic037_cp1_cs_as', False),
-('SQL_EBCDIC273_CP1_CS_AS', 'sql_ebcdic273_cp1_cs_as', False),
-('SQL_EBCDIC277_CP1_CS_AS', 'sql_ebcdic277_cp1_cs_as', False),
-('SQL_EBCDIC278_CP1_CS_AS', 'sql_ebcdic278_cp1_cs_as', False),
-('SQL_EBCDIC280_CP1_CS_AS', 'sql_ebcdic280_cp1_cs_as', False),
-('SQL_EBCDIC284_CP1_CS_AS', 'sql_ebcdic284_cp1_cs_as', False),
-('SQL_EBCDIC285_CP1_CS_AS', 'sql_ebcdic285_cp1_cs_as', False),
-('SQL_EBCDIC297_CP1_CS_AS', 'sql_ebcdic297_cp1_cs_as', False)
+MSSQL_COLLATION_MAP = [
+    ('SQL_Latin1_General_Cp437_BIN', 'utf8_general_ci', False),
+    ('SQL_Latin1_General_Cp437_CS_AS', 'utf8_general_ci', False),
+    ('SQL_Latin1_General_Cp437_CI_AS', 'utf8_general_ci', False),
+    ('SQL_Latin1_General_Pref_CP437_CI_AS', 'utf8_general_ci', False),
+    ('SQL_Latin1_General_Cp437_CI_AI', 'utf8_general_ci', False),
+    ('SQL_Latin1_General_Cp850_BIN', 'utf8_bin', True),
+    ('SQL_Latin1_General_Cp850_CS_AS', 'utf8_general_ci', False),
+    ('SQL_Latin1_General_Cp850_CI_AS', 'utf8_general_ci', False),
+    ('SQL_Latin1_General_Pref_CP850_CI_AS', 'utf8_general_ci', False),
+    ('SQL_Latin1_General_Cp850_CI_AI', 'utf8_general_ci', False),
+    ('SQL_1Xcompat_CP850_CI_AS', 'utf8_general_ci', False),
+    ('Latin1_General_BIN', 'latin1_bin', True),
+    ('SQL_Latin1_General_Cp1_CS_AS', 'utf8_general_ci', False),
+    ('SQL_Latin1_General_Cp1_CI_AS', 'utf8_general_ci', True),
+    ('SQL_Latin1_General_Pref_CP1_CI_AS', 'utf8_general_ci', False),
+    ('SQL_Latin1_General_Cp1_CI_AI', 'utf8_general_ci', True),
+    ('SQL_AltDiction_Cp850_CS_AS', 'utf8_general_ci', False),
+    ('SQL_AltDiction_Pref_CP850_CI_AS', 'utf8_general_ci', False),
+    ('SQL_AltDiction_Cp850_CI_AI', 'utf8_general_ci', False),
+    ('SQL_Scandinavian_Pref_Cp850_CI_AS', 'utf8_swedish_ci', False),
+    ('SQL_Scandinavian_Cp850_CS_AS', 'utf8_swedish_ci', False),
+    ('SQL_Scandinavian_Cp850_CI_AS', 'utf8_swedish_ci', False),
+    ('SQL_AltDiction_Cp850_CI_AS', 'utf8_general_ci', False),
+    ('Latin1_General_CS_AS', 'utf8_general_ci', False),
+    ('Latin1_General_CI_AS', 'utf8_general_ci', False),
+    ('Danish_Norwegian_CS_AS', 'utf8_danish_ci', False),
+    ('Finnish_Swedish_CS_AS', 'utf8_swedish_ci', False),
+    ('Icelandic_CS_AS', 'utf8_icelandic_ci', False),
+    ('Hungarian_BIN', 'utf8_bin', True),
+    ('SQL_Latin1_General_Cp1250_CS_AS', 'cp1250_general_ci', False),
+    ('SQL_Latin1_General_Cp1250_CI_AS', 'cp1250_general_ci', True),
+    ('SQL_Czech_Cp1250_CS_AS', 'cp1250_czech_cs', True),
+    ('SQL_Czech_Cp1250_CI_AS', 'cp1250_czech_cs', False),
+    ('SQL_Hungarian_Cp1250_CS_AS', 'cp1250_general_ci', False),
+    ('SQL_Hungarian_Cp1250_CI_AS', 'cp1250_general_ci', False),
+    ('SQL_Polish_Cp1250_CS_AS', 'sql_polish_cp1250_cs_as', False),
+    ('SQL_Polish_Cp1250_CI_AS', 'sql_polish_cp1250_ci_as', False),
+    ('SQL_Romanian_Cp1250_CS_AS', 'sql_romanian_cp1250_cs_as', False),
+    ('SQL_Romanian_Cp1250_CI_AS', 'sql_romanian_cp1250_ci_as', False),
+    ('SQL_Croatian_Cp1250_CS_AS', 'sql_croatian_cp1250_cs_as', False),
+    ('SQL_Croatian_Cp1250_CI_AS', 'sql_croatian_cp1250_ci_as', False),
+    ('SQL_Slovak_Cp1250_CS_AS', 'sql_slovak_cp1250_cs_as', False),
+    ('SQL_Slovak_Cp1250_CI_AS', 'sql_slovak_cp1250_ci_as', False),
+    ('SQL_Slovenian_Cp1250_CS_AS', 'sql_slovenian_cp1250_cs_as', False),
+    ('SQL_Slovenian_Cp1250_CI_AS', 'sql_slovenian_cp1250_ci_as', False),
+    ('Cyrillic_General_BIN', 'cyrillic_general_bin', False),
+    ('SQL_Latin1_General_Cp1251_CS_AS', 'sql_latin1_general_cp1251_cs_as', False),
+    ('SQL_Latin1_General_Cp1251_CI_AS', 'sql_latin1_general_cp1251_ci_as', False),
+    ('SQL_Ukrainian_Cp1251_CS_AS', 'sql_ukrainian_cp1251_cs_as', False),
+    ('SQL_Ukrainian_Cp1251_CI_AS', 'sql_ukrainian_cp1251_ci_as', False),
+    ('Greek_BIN', 'greek_bin', False),
+    ('SQL_Latin1_General_Cp1253_CS_AS', 'sql_latin1_general_cp1253_cs_as', False),
+    ('SQL_Latin1_General_Cp1253_CI_AS', 'sql_latin1_general_cp1253_ci_as', False),
+    ('SQL_MixDiction_Cp1253_CS_AS', 'sql_mixdiction_cp1253_cs_as', False),
+    ('SQL_AltDiction_Cp1253_CS_AS', 'sql_altdiction_cp1253_cs_as', False),
+    ('SQL_Latin1_General_Cp1253_CI_AI', 'sql_latin1_general_cp1253_ci_ai', False),
+    ('Turkish_BIN', 'turkish_bin', False),
+    ('SQL_Latin1_General_Cp1254_CS_AS', 'sql_latin1_general_cp1254_cs_as', False),
+    ('SQL_Latin1_General_Cp1254_CI_AS', 'sql_latin1_general_cp1254_ci_as', False),
+    ('Hebrew_BIN', 'hebrew_bin', False),
+    ('SQL_Latin1_General_Cp1255_CS_AS', 'sql_latin1_general_cp1255_cs_as', False),
+    ('SQL_Latin1_General_Cp1255_CI_AS', 'sql_latin1_general_cp1255_ci_as', False),
+    ('Arabic_BIN', 'arabic_bin', False),
+    ('SQL_Latin1_General_Cp1256_CS_AS', 'sql_latin1_general_cp1256_cs_as', False),
+    ('SQL_Latin1_General_Cp1256_CI_AS', 'sql_latin1_general_cp1256_ci_as', False),
+    ('SQL_Latin1_General_Cp1257_CS_AS', 'sql_latin1_general_cp1257_cs_as', False),
+    ('SQL_Latin1_General_Cp1257_CI_AS', 'sql_latin1_general_cp1257_ci_as', False),
+    ('SQL_Estonian_Cp1257_CS_AS', 'sql_estonian_cp1257_cs_as', False),
+    ('SQL_Estonian_Cp1257_CI_AS', 'sql_estonian_cp1257_ci_as', False),
+    ('SQL_Latvian_Cp1257_CS_AS', 'sql_latvian_cp1257_cs_as', False),
+    ('SQL_Latvian_Cp1257_CI_AS', 'sql_latvian_cp1257_ci_as', False),
+    ('SQL_Lithuanian_Cp1257_CS_AS', 'sql_lithuanian_cp1257_cs_as', False),
+    ('SQL_Lithuanian_Cp1257_CI_AS', 'sql_lithuanian_cp1257_ci_as', False),
+    ('SQL_Danish_Pref_Cp1_CI_AS', 'sql_danish_pref_cp1_ci_as', False),
+    ('SQL_SwedishPhone_Pref_Cp1_CI_AS', 'sql_swedishphone_pref_cp1_ci_as', False),
+    ('SQL_SwedishStd_Pref_Cp1_CI_AS', 'sql_swedishstd_pref_cp1_ci_as', False),
+    ('SQL_Icelandic_Pref_Cp1_CI_AS', 'sql_icelandic_pref_cp1_ci_as', False),
+    ('Japanese_BIN', 'japanese_bin', False),
+    ('Japanese_CI_AS', 'japanese_ci_as', False),
+    ('Korean_Wansung_BIN', 'korean_wansung_bin', False),
+    ('Korean_Wansung_CI_AS', 'korean_wansung_ci_as', False),
+    ('Chinese_Taiwan_Stroke_BIN', 'chinese_taiwan_stroke_bin', False),
+    ('Chinese_Taiwan_Stroke_CI_AS', 'chinese_taiwan_stroke_ci_as', False),
+    ('Chinese_PRC_BIN', 'chinese_prc_bin', False),
+    ('Chinese_PRC_CI_AS', 'chinese_prc_ci_as', False),
+    ('Japanese_CS_AS', 'japanese_cs_as', False),
+    ('Korean_Wansung_CS_AS', 'korean_wansung_cs_as', False),
+    ('Chinese_Taiwan_Stroke_CS_AS', 'chinese_taiwan_stroke_cs_as', False),
+    ('Chinese_PRC_CS_AS', 'chinese_prc_cs_as', False),
+    ('Thai_BIN', 'thai_bin', False),
+    ('Thai_CI_AS', 'thai_ci_as', False),
+    ('Thai_CS_AS', 'thai_cs_as', False),
+    ('SQL_EBCDIC037_CP1_CS_AS', 'sql_ebcdic037_cp1_cs_as', False),
+    ('SQL_EBCDIC273_CP1_CS_AS', 'sql_ebcdic273_cp1_cs_as', False),
+    ('SQL_EBCDIC277_CP1_CS_AS', 'sql_ebcdic277_cp1_cs_as', False),
+    ('SQL_EBCDIC278_CP1_CS_AS', 'sql_ebcdic278_cp1_cs_as', False),
+    ('SQL_EBCDIC280_CP1_CS_AS', 'sql_ebcdic280_cp1_cs_as', False),
+    ('SQL_EBCDIC284_CP1_CS_AS', 'sql_ebcdic284_cp1_cs_as', False),
+    ('SQL_EBCDIC285_CP1_CS_AS', 'sql_ebcdic285_cp1_cs_as', False),
+    ('SQL_EBCDIC297_CP1_CS_AS', 'sql_ebcdic297_cp1_cs_as', False)
 ]
 
 truncated_identifier_serial = 0
@@ -146,8 +146,8 @@ class MSSQLMigration(GenericMigration):
             mysql_name = mssql_name[1:-1]
         if not dots_allowed and not mysql_valid_regex.match(mysql_name):
             mysql_name = mysql_name.translate(string.maketrans(r'/\.', '___'))
-            log_message = (u'Schema and table names cannot contain "/", "\\", ".", or characters that are '
-                           u'not permitted in file names. The identifier [%s] was changed to `%s`. ' % (mssql_name, mysql_name))
+            log_message = ('Schema and table names cannot contain "/", "\\", ".", or characters that are '
+                           'not permitted in file names. The identifier [%s] was changed to `%s`. ' % (mssql_name, mysql_name))
         
         # truncate too long identifiers
         if len(mysql_name) > 64:
@@ -155,7 +155,7 @@ class MSSQLMigration(GenericMigration):
             truncated_identifier_serial += 1
             original_mysql_name = mysql_name
             mysql_name = mysql_name[:62]+str(truncated_identifier_serial)
-            log_message += u'Identifier `%s` is too long, truncated to `%s`' % (original_mysql_name, mysql_name)
+            log_message += 'Identifier `%s` is too long, truncated to `%s`' % (original_mysql_name, mysql_name)
 
         if log and log_message:
             entry = grt.classes.GrtLogEntry()
@@ -248,8 +248,6 @@ class MSSQLMigration(GenericMigration):
             elif source_datatype == 'UNIQUEIDENTIFIER':
                 target_datatype = 'VARCHAR'
                 target_column.length = 64
-                if 'UNIQUE' not in target_column.flags:
-                    target_column.flags.append('UNIQUE') # uniqueid must be UNIQUE... bug #43098
                 state.addMigrationLogEntry(0, source_column, target_column,
                         "Source column type %s was migrated to %s(%s)" % (source_datatype, target_datatype, target_column.length))
             elif source_datatype == 'SYSNAME':  # the relevant info is in http://msdn.microsoft.com/en-us/library/ms191240(v=sql.105).aspx
@@ -328,7 +326,7 @@ class MSSQLMigration(GenericMigration):
                 # just fall back to same type name and hope for the best
                 target_datatype = source_datatype
 
-            if mysql_simpleTypes.has_key(target_datatype):
+            if target_datatype in mysql_simpleTypes:
                 target_column.simpleType = mysql_simpleTypes[target_datatype]
             else:
                 grt.log_warning("Migration", "MSSQL migrateTableColumnsToMySQL", "Can't find datatype %s for type %s\n" % (target_datatype, source_datatype))
